// ==UserScript==
// @name PwdHash for Opera
// @author Patrick Schmidt and Haris Andrianakis
// @version 1.1
// ==/UserScript==

/*
 *   PwdHash for Opera 1.1 (2011-03-09)
 *                                         Copyright (C) 2005 by Patrick Schmidt
 *                                                       2011 by Haris Andrianakis
 *
 *
 *  A User JavaScript implementation of PwdHash for use in Opera, reusing
 *  existing code from Remote PwdHash as libraries (for domain extraction and
 *  the actual password hashing). This script "only" adds the integration into
 *  Opera.
 *  Please note that I only tested it with Opera 11.01 for Linux. So any
 *  comments on whether and how it works with different versions of Opera
 *  would be helpful.
 *
 *
 *  !! Please be aware that (being just a JavaScript) this implementation can't
 *  protect you against phishing like the plug-ins for Internet Explorer and
 *  FireFox can. Any malicious page could compromise this User Script by using
 *  JavaScript itself. !!
 *
 *
 ** Sources for more information:   ********************************************
 *
 *   (Remote) PwdHash   --   http://crypto.stanford.edu/PwdHash/
 *   User JavaScript    --   http://people.opera.com/rijk/opera/userjs.html
 *
 *
 ** Requirements:   ************************************************************
 *
 *   In order to use User JavaScripts (like this one) you need Opera 11.01 or later.
 *
 *
 ** Installation:   ************************************************************
 *
 *   First you have to copy the files to a location where they can stay. I
 *   suggest somewhere within your Opera folder, e.g.:
 *
 *     <YourOperaFolder>\userjs\PwdHash\
 *
 *   Now you have to open Opera's Preferences dialog and select the Advanced
 *   tab. In the left list select 'Content' and there press the 'JavaScript
 *   options...' button. At the bottom of the dialog is a text field labeled
 *   'My JavaScript files'. Here you have to enter the path to those JavaScript
 *   files. You should enter an absolute path because relative paths won't
 *   always work, e.g. when you start Opera by double clicking an html file.
 *   It's enough to enter the directory (instead of listing every single file).
 *   Opera will automatically scan it for JavaScripts. According to the
 *   suggested directory above you would enter: <YourOperaFolder>\userjs\PwdHash
 *
 *
 ** Usage:   *******************************************************************
 *
 *   This implementation works mostly as described at
 *   http://crypto.stanford.edu/PwdHash/. To activate the hashing you either
 *   have to prepend your password with '@@' (two ats) or press F6 (instead of
 *   F2) while the password field is focused. When hashing is activated the
 *   background of the password field will turn to a very bright yellow and
 *   a yellow hash sign will appear at the bottom right of the field. Once
 *   you leave the password field or press enter to submit the form the password
 *   will be hashed. This is indicated by a bright green background and a green
 *   hash sign in the field.
 *   You can deactivate the hashing by pressing F6 again which will also restore
 *   the password field's original background color.
 *   Hashing can even be activated for regular text fields.
 *
 *
 ** Customization:   ***********************************************************
 *
 *   I decided to use F6 (unlike the plugins for FireFox and the Internet
 *   Explorer which both use F2) because its the first function key in Opera
 *   without any pre-assigned action. So you don't have to meddle with your
 *   keyboard shortcuts.
 *   If you really want to use another function key, change the following code
 *   fragment: 'event.keyCode == 117'. 117 corresponds to F6, 116 to F5...
 *   I think you can figure out the rest by yourself. (Unless there is some
 *   inconsistency. I haven't tried all the keyCodes. If there is, let me know.)
 *
 *   The following variables allow you to customize the behavior of this script:
 */

(function(){
 // If this variable is set to 'true' hashing will automatically be activated for
 // all password fields. Remember that you can deactivate the hashing again for
 // individual fields by pressing F6 while they are focused. Set the variable to
 // 'false' to turn this feature off.
var PH_defaultHashAllPasswordFields = false;
	
 // If this variable is set to 'true' the background color will change to
 // indicate what's going on (see "Usage" for more details). Set the variable
 // to 'false' to turn this feature off.
var PH_changeBackgroundColor = true;
 // If this variable is set to 'true' background images (hash signs) will be used
 // to indicate what's going on (see "Usage" for more details). Set the variable
 // to 'false' to turn this feature off.
var PH_changeBackgroundImage = false;

 // This is the background color used to indicate that hashing has been activated
 // for a password field.
var PH_activatedBackgroundColor = "#FFF5C3";
 // This is the background color used to indicate that hashing has been completed
 // for a password field.
var PH_hashedBackgroundColor = "#B0FD93";

 // This is the background image used to indicate that hashing has been activated
 // for a password field.
var PH_activatedBackgroundImage = "http://codecamp.de/pwdhashforopera/hashingActivated.png";
 // This is the background image used to indicate that hashing has been completed
 // for a password field.
var PH_hashedBackgroundImage = "http://codecamp.de/pwdhashforopera/hashingComplete.png";

/*
 *
 ** Known issues:   ************************************************************
 *
 *   - If you submit a form by pressing enter/return while still in the
 *     password field and you tell Opera to save the password, it will save
 *     the unhashed password. This is because onSubmit events are processed
 *     AFTER that dialog. And as far as I know there is no other way to catch
 *     this case. A workaround for this (luckily rare) case is to unfocus the
 *     password field, i.e. clicking anywhere outside the password field (e.g.
 *     the login button) or pressing tab.
 *
 *   - UserJS won't be executed on secured pages (https://) and therefore this
 *     script won't work either. And as far as I know there's no way around
 *     this. But of course you still have the option to use Remote PwdHash.
 *
 *     UPDATE: On Opera version 11.01 it is possible to enable userjs on secured
 *     pages. Just follow the instructions in this page:
 *     http://www.opera.com/docs/userjs/using/#securepages
 *
 *   - If you want to use different background images you have to host them on a
 *     webserver. 'file://' won't work properly, probably for security reasons.
 *     So the default images are hosted on "my" server for the moment. As the
 *     images aren't very big (1,13KB altogether) I'm somewhat confident that
 *     this won't get my available transfer volume into too much trouble.
 *     But you're still encouraged to host them on you own webspace. If you do
 *     you have to edit the associated variables to reflect their new location
 *     (see 'Customization' section).
 *
 *
 ** Version History:   *********************************************************
 *
 *   1.1 (2011-03-09) ----------
 *
 *     Compatible with Opera 11.01
 *
 *   1.0 (2005-09-30) ----------
 *
 *     The first release.
 *
 *
 ** Special Thanks:   **********************************************************
 *
 *   Special thanks to Boris Fresow for helping me figure out some tricky
 *   JavaScript related issues and dealing with my constant flow of questions.
 *
 *
 ** Contact:   *****************************************************************
 *
 *   Comments, feature requests, bug reports, etc. are welcome.
 *
 *   eMail   --   operapwdhash@codecamp.de
 *   Web     --   http://codecamp.de/pwdhashforopera/
 *
 */



/*******************************************************************************
 * Here starts the actual script. You shouldn't change anything below this point
 * unless you know what you're doing.
 */


var PH_status = new Object();
var PH_originalBackgroundColor = new Object();
var PH_focusedField = null;
var PH_focusedFieldLength = 0;

function PH_GetElementId( inputElement )
{
	return inputElement.form.name + "." + inputElement.name;
}
function PH_IsPrefixedPassword( password )
{
	return (password.substring( 0, SPH_kPasswordPrefix.length ) == SPH_kPasswordPrefix);
}

function PH_SetBackgroundColor( inputElement, color )
{
	if( PH_changeBackgroundColor )
	{
		if( color == "" )
		{
			inputElement.style.backgroundColor = "";
		}
		else
			inputElement.style.backgroundColor = color;
	}
}
function PH_SetBackgroundImage( inputElement, image )
{
	if( PH_changeBackgroundImage )
	{
		if( image == null )
		{
			inputElement.style.backgroundImage = "";
		}
		else
		{
			inputElement.style.backgroundRepeat = "no-repeat";
			inputElement.style.backgroundPosition = "bottom right";
			inputElement.style.backgroundImage = "url( " + image + " )";
		}
	}
}

function PH_SetStatusFor( inputElement, status )
{
	if( status == 0 )
	{
		PH_status[PH_GetElementId( inputElement )] = 0;
		PH_SetBackgroundImage( inputElement, null );
		PH_SetBackgroundColor( inputElement, PH_originalBackgroundColor[PH_GetElementId( inputElement )] );
	}
	else if( status == 1 )
	{
		PH_status[PH_GetElementId( inputElement )] = 1;
		PH_SetBackgroundImage( inputElement, PH_activatedBackgroundImage );
		PH_SetBackgroundColor( inputElement, PH_activatedBackgroundColor );
	}
	else if( status == 2 )
	{
		PH_status[PH_GetElementId( inputElement )] = 2;
		PH_SetBackgroundImage( inputElement, PH_hashedBackgroundImage );
		PH_SetBackgroundColor( inputElement, PH_hashedBackgroundColor );
	}
}
function PH_GetStatusFor( inputElement )
{
	return PH_status[PH_GetElementId( inputElement )];
}


function PH_HashPassword( inputElement )
{
	var data = inputElement.value;

	if( data.length == 0 )
		return;

	if( (inputElement.type == "password" && PH_IsPrefixedPassword( data ))
	    || PH_status[PH_GetElementId( inputElement )] == true )
	{
		var url = location.href;
		var domain = (new SPH_DomainExtractor()).extractDomain( url );
		if( PH_IsPrefixedPassword( data ) )
			data = data.substring( SPH_kPasswordPrefix.length );
		var result = new String(new SPH_HashedPassword( data, domain ));
	
		inputElement.value = result;

		PH_SetStatusFor( inputElement, 2 );
	}
}


function PH_ChangeListener( event )
{
	PH_HashPassword( event.srcElement );
}
function PH_KeyPressListener( event )
{
	var inputElement = event.srcElement;

	if( event.which == 0 && event.keyCode == 117 )
	{
		if( PH_GetStatusFor( inputElement ) > 0 )
			PH_SetStatusFor( inputElement, 0 );
		else
			PH_SetStatusFor( inputElement, 1 );
	}
}
function PH_KeyUpListener( event )
{
	var inputElement = event.srcElement;

	if( PH_GetStatusFor( inputElement ) > 0 )
	{
		if( PH_focusedFieldLength != inputElement.value.length )
			PH_SetStatusFor( inputElement, 1 );
	}
	else
	{
		if( inputElement.type == "password" && PH_IsPrefixedPassword( inputElement.value ) )
			PH_SetStatusFor( inputElement, 1 );
	}
}
function PH_FocusGained( event )
{
	PH_focusedField = event.srcElement;
	PH_focusedFieldLength = event.srcElement.value.length;
}
function PH_FocusLost( event )
{
	PH_focusedField = null;

	var inputElement = event.srcElement;

	if( PH_GetStatusFor( inputElement ) == 1 )
		PH_HashPassword( inputElement );
}
function PH_SubmitListener( event )
{
	if( PH_focusedField != null )
		PH_HashPassword( PH_focusedField );
}


addEventListener('load', function(event) {
    if (!document.body)
    {
        return;
    }

	for (var f = 0; f < document.forms.length; f++)
	{
		var form = document.forms[f];
		form.addEventListener( "submit", PH_SubmitListener, false );
		for( var e = 0; e < form.length; e++ )
		{
			var element = form.elements[e];
			if( element.type == "text" || element.type == "password" )
			{
				element.addEventListener( "change", PH_ChangeListener, false );
				element.addEventListener( "keypress", PH_KeyPressListener, false );
				element.addEventListener( "keyup", PH_KeyUpListener, false );
				element.addEventListener( "focus", PH_FocusGained, false );
				element.addEventListener( "blur", PH_FocusLost, false );
				PH_originalBackgroundColor[PH_GetElementId( element )] = element.style.backgroundColor;
				PH_SetStatusFor( element, 0 );
			}
			if( PH_defaultHashAllPasswordFields == true && element.type == "password" )
			{
				PH_SetStatusFor( element, 1 );
			}
		}
	}

	// These three lines of code ensure that both images are properly preloaded.
	// I noticed some minor graphic glitches without them.
	// var bild = new Image();
	// bild.src = PH_hashedBackgroundImage;
	// bild.src = PH_activatedBackgroundImage

}, false );
})();
